from margot import BackTest


class Strategy(object):
 
    def __init__ (self, algo, recent_vol, target_vol):   # noqa: D107
        self.algo = algo
        self.recent_vol = recent_vol
        self.target_vol = target_vol

    def vol_size(self):
        return self.recent_vol / self.target_vol


class Portfolio(object):

    def __init__ (self, account_size, target_vol):  # noqa: D107
        self.account_size = account_size
        self.target_vol = target_vol
        self.strategies = list()

    def add_strategy(self, algo, target_vol):
        """Add a strategy to the portfolio.

        Args:
            algo (Algo): A Margot Trading Algorithm
            target_vol (float): the vol were aiming for
        """
        bt = BackTest(algo)

        #TODO: periods assumes we're looking at days.
        bt.run(periods=30)  

        self.strategies.append(
            Strategy(algo, bt.volatility(), target_vol)
            )
        


from margot import Symbol, MargotDataFrame, Ratio, BaseAlgo, Position, BackTest
from margot import finance, cboe, alphavantage as av

def test_algo():

    class Index(Symbol):
        close = cboe.Column(time_series='close')

    class VXBasis(MargotDataFrame):
        vixm = Index(symbol='VIX', trading_calendar='NYSE')

    class MyAlgo(BaseAlgo):
        data = MargotDataFrame()

    myalgo = MyAlgo(env={})



from margot.data.frames import MargotDataFrame
from margot.data.symbols import Symbol
from margot.data.ratio import Ratio
from margot.data.column import alphavantage, cboe
from margot.data.features import finance
from margot.signals.algos import BaseAlgo
from margot.signals.backtest import BackTest
from margot.signals import Position


from ._version import get_versions
__version__ = get_versions()['version']
del get_versions




from datetime import datetime

import pytz

from margot import Symbol, MargotDataFrame, Ratio, BaseAlgo, Position, BackTest
from margot import finance, cboe, alphavantage as av


class Index(Symbol):
    close = cboe.Column(time_series='close')
    sma = finance.SimpleMovingAverage(column='close',
                                      window=3)


class Equity(Symbol):
    close = av.Column(time_series='adjusted_close')
    simple_returns = finance.SimpleReturns(column='close')


class VXBasisDF(MargotDataFrame):
    vix = Index('VIX', trading_calendar='NYSE')
    vix3m = Index('VIX3M', trading_calendar='NYSE')
    ratio = Ratio(numerator=vix.close,
                  denominator=vix3m.close,
                  label='vxbasis')
    ziv = Equity('ZIV', trading_calendar='NYSE')


class VXBasisTrade(BaseAlgo):
    data = VXBasisDF()

    def signal(self):
        if self.data.ratio.latest <= 1.0 and \
            self.data.vix.close.latest <= self.data.vix.sma.latest:
            return [Position('ZIV', 1.0, self.MOC)]
        else:
            return [Position('ZIV', 0.0, self.MOC)]

                    
def test_simulation():
    vxa = VXBasisDF()
    vxb = vxa.to_pandas().copy()
    when = datetime(2020, 1, 1, tzinfo=pytz.UTC)
    vxa.simulate(when)
    assert(vxb[:when].equals(vxa.to_pandas()))


def test_backtest(): 
    vx = VXBasisTrade()   
    bt = BackTest(vx)
    bt.run(periods=100)
